#include <Servo.h>  

Servo CameraServo;    // Der Servo wird als Variable deklariert
int servoState = 0;   // servoState gibt an in welche Richtung sich der Servo bewegen soll
int servoAngle = 90;  // servoAngle gibt den Winkel an in dem sich der Servo befindet
int servoDelay = 30;  // gibt an wie lange das delay istt zwischen dem ändern vom Winkel
unsigned long previousServoMillis = 0;  // previousServoMillis gibt die letzten zwischengespeicherten Milisekunden für das Servo-delay an

int turnArmState = 0;   // turnArmState gibt an ob die Basis des Arms sich bewegt
int turnArmDelay = 20;  // ist das delay für eine bewegung des Arms auf der Basis
unsigned long previousArmMillis = 0;    // previousArmMillis gibt die letzten zwischengespeicherten Milisekunden für das turnArmDelay an
int turnArmFlipFlop = 0;  // turnArmFlipFlop dient als flipFlop für das switchen zwischen HIGH und LOW beim bewegen von dem Stepper-Motor

int verticalArmState = 0; // verticalArmState gibt an ob der Arms sich hoch oder runter bewegt
int vTurnArmDelay = 40;   // vTurnArmDelay ist das delay zwischen einzelnen bewegungen vom Arm beim hoch und Runter bewegen
unsigned long previousVArmMillis = 0;   // previousVArmMillis gibt die letzten zwischengespeicherten Milisekunden für das vTurnArmDelay an
int VArmFlipFlop = 0;     // VArmFlipFlop dient als flipFlop für das switchen zwischen HIGH und LOW beim bewegen von dem Stepper-Motor

int platformState = 0;        // platformState gibt an ob die Platform sich bewegen soll
int turnPlatformDelay = 50;   // turnPlatformDelay ist das delay zwischen den einzelnen Steps des Stepper-Motors an der Achse
unsigned long previousPlatformMillis = 0;   // previousPlatformMillis gibt die letzten zwischengespeicherten Milisekunden für das turnPlatformDelay an
int platformFlipFlop = 0;     // platformFlipFlop dient als flipFlop für das switchen zwischen HIGH und LOW beim bewegen von dem Stepper-Motor
 
const unsigned int messageLength = 7;   // gibt die lenge des Strings an der Empfangen werden soll
unsigned long currentMillis = 0;        // currentMillis gibt später die anzahl an vergangenen Milisekunden an

void setup() {
  CameraServo.attach(8);  // Der Kamera Servo wird mit dem Pin 8 verbunden
  Serial.begin(9600);     // Die Baud rate wird auf 9600 gelegt
  pinMode(2, OUTPUT);     // Pin 2-7 werden als Output Pin festgelegt
  pinMode(3, OUTPUT);     
  digitalWrite(3, LOW);
  pinMode(4, OUTPUT);
  pinMode(5, OUTPUT);
  digitalWrite(5, LOW);
  pinMode(6, OUTPUT);
  pinMode(7, OUTPUT);
  digitalWrite(7, LOW); 
}

void loop() {
  currentMillis = millis();       // currentMillis wird gleich mit den vergangenen Millisekunden die das Programm läuft gesetzt
  while (Serial.available()> 0)   // solange es eine Nachricht zu lesen gibt wird die while Schleife ausgeführt
   {
      static char message[messageLength];       // Array message für die gesendete Nachricht wird definiert
      static unsigned int messagePosition = 0;  // messagePosition gibt die Position an in die bei dem message Array geschrieben werden soll
      char inByte = Serial.read();              // Speichert den nächsten gelesenen Byte in der Variable inByte
      
      if (inByte != '\n'){                  // solange das ende der Nachricht nicht erreicht ist wird der zulestzt gespeicherte Zwischenwert dem Array hinzugefügt
        message[messagePosition] = inByte;
        messagePosition++;
      } else {                              // wenn das ende der Nachricht erreeicht ist werden die einzelnen Variablen nach dem Array bestimmt
        message[messagePosition] = '\0';
        
        Serial.println(message);
        turnArmState = message[0] - '0';          // der erste Ziffer der Nachricht bestimmt ob der Arm sich drehen soll
        int armDirection = message[1] - '0';      // die zweite Ziffer der Nachricht bestimmt in welche Richtung er sich drehen soll
        setDirection(armDirection, 3);           

        verticalArmState = message[2] - '0';            // die dritte Ziffer sag ob sich der Arm Auf oder Ab bewegen soll
        int verticalArmDirection = message[3] - '0';    // die Vierte Ziffer entscheidet wieder die Richtung
        setDirection(verticalArmDirection, 5);

        platformState = message[4] - '0';             // die fünfte Ziffer sagt ob sich die Platform bewegen soll
        int platformDirection = message[5] - '0';     // die sechste Ziffer sagt die Richtung der bewegung der Platform
        setDirection(platformDirection, 7);

        servoState = message[6]  - '0';     // die Servo bewegung wird aus der siebten Ziffer Ermittelt
        
        messagePosition = 0;      // die message position wird für den nächsten Umlauf auf 0 zurückgesetzt
      }
   }
   turnArm();           // ruft die Funktion auf die den Arm an der Basis dreht
   verticalTurnArm();   // ruft die Funktion auf welche den Arm auf und ab bewegt
   movePlatform();      // ruft die Funktion auf die die Platform bewegt
   turnCamera();        // ruft die Funktion auf die die Kamera dreht      
}

// setzt die Direction von bestimmten Steppermotoren, wenn die gegeben Direction 0 wird der angesteurte Pin auf Low gesezt, bei anderen weten auf HIGH
 void setDirection(int Direction, int Pin){
   if (Direction == 0) {
     digitalWrite(Pin, LOW);
   } else {
     digitalWrite(Pin, HIGH);
   }
 }

// Funktion die den Arm auf der Basis hin und her Rotiert
 void turnArm() {
  if (turnArmState == 1) {    // wenn turnArmState = 1 wird der Arm bewegt
    if (currentMillis - previousArmMillis >= turnArmDelay){   // schaut ob das Delay erreicht ist
      previousArmMillis = currentMillis;
      if (turnArmFlipFlop == 0){      // flipFlop entscheidet abwechselnd zwischen HIGH und LOW
        digitalWrite(2, HIGH);
        turnArmFlipFlop = 1;
        } else {
        digitalWrite(2, LOW);
        turnArmFlipFlop = 0;
      }
    }
  }
}

// Funktion die den Arm nach oben und nach unten bewegt
 void verticalTurnArm (){
  if (verticalArmState == 1) {    // wenn verticalArmState = 1 wird der Arm bewegt
    if (currentMillis - previousVArmMillis >= vTurnArmDelay){   // schaut ob das Delay erreicht ist
      previousVArmMillis = currentMillis;
      if (VArmFlipFlop == 0){        // flipFlop entscheidet abwechselnd zwischen HIGH und LOW
         digitalWrite(4, HIGH);
         VArmFlipFlop = 1;
       } else {
         digitalWrite(4, LOW);
         VArmFlipFlop = 0;
       }
     }
   }
 }

// Funktion welchen den Stepper-Motor unter der Platform ansteuert und die Platform hin und her bewegt
 void movePlatform (){
  if (platformState == 1){
    if (currentMillis - previousPlatformMillis >= turnPlatformDelay){
      previousPlatformMillis = currentMillis;
      if (platformFlipFlop == 0){         // flipFlop entscheidet abwechselnd zwischen HIGH und LOW
        digitalWrite(6,HIGH);
        platformFlipFlop = 1;
      } else {
        digitalWrite(6,LOW);
        platformFlipFlop = 0;
      }
    }
  }
}

// Funktion die das Servo welches die Kamera dreht hin und her bewegt
void turnCamera (){
  if (servoState == 1){           // wenn servoState gleich 1 wird der Servo in die eine Richtung gedreht 
    if (servoAngle > 30 && currentMillis - previousServoMillis >= servoDelay){      // Es wird geschaut ob genug Zeit für das delay vergangen ist
      previousServoMillis = currentMillis;
      servoAngle--;
      CameraServo.write(servoAngle);
    }
  }
  else if (servoState == 2){     // wenn servoState gleich 2 wird der Servo in die andere Richtung gedreht
    if (servoAngle < 150 && currentMillis - previousServoMillis >= servoDelay){       // Es wird geschaut ob genug Zeit für das delay vergangen ist
      previousServoMillis = currentMillis;
      servoAngle++;
      CameraServo.write(servoAngle);
    }
  }
}
